/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2010 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include "AppleContactItem.h"
#include "spds/spdsutils.h"
#include "base/base64.h"
#import <Foundation/Foundation.h>

using namespace std;
using namespace Funambol;

AppleContactItem::AppleContactItem()
{
    contactProperties.clear();
}

AppleContactItem::AppleContactItem(const char* vCard)
{
    initFromVCard(vCard);
}

bool AppleContactItem::initFromVCard(const char* vCard)
{
	int businessTel     = 0;
	int homeTel		    = 0;
    int imAddress       = 0;
	const char* name    = NULL;
    const char* element = NULL;

    if (vCard == NULL) {
        LOG.error("%s: invalid parameter", __FUNCTION__);

        return  false;
    }

    // parse the vCard and fill the VObject.
    VObject* vo = VConverter::parse(vCard);
    if (!vo) {
        return false;
    }

    // conversion vObject -> AppleContactItem
    contactProperties.clear();

    for(int i=0; i < vo->propertiesCount(); i++) {
        VProperty* vp = vo->getProperty(i);
		name    = vp->getName();
        element = vp->getValue(0);      // the first value of the property.

		if (strcmp(name, "N") == 0) {
			if (element = vp->getPropComponent(1))  
                contactProperties[APPLE_CONTACT_FAMILY_NAME] = element;
			if (element = vp->getPropComponent(2))  
                contactProperties[APPLE_CONTACT_GIVEN_NAME] = element;
			if (element = vp->getPropComponent(3))  
                contactProperties[APPLE_CONTACT_ADDITIONAL_NAME] = element;
			if (element = vp->getPropComponent(4))  
                contactProperties[APPLE_CONTACT_NAME_PREFIXES] = element;
			if (element = vp->getPropComponent(5))  
                contactProperties[APPLE_CONTACT_NAME_SUFFIXES] = element;
		} else if (strcmp(name, "BDAY") == 0) {
			contactProperties[APPLE_CONTACT_BIRTH_DATE] = element;
        } else if (strcmp(name, "NOTE") == 0) {
			contactProperties[APPLE_CONTACT_NOTE] = element;
        } else if (strcmp(name, "CATEGORIES") == 0) {
			contactProperties[APPLE_CONTACT_CATEGORIES] = element;
        } else if (strcmp(name, "EMAIL") == 0) {
            // Mapping is:
            // Email1Address <-> EMAIL;INTERNET:
            // Email2Address <-> EMAIL;INTERNET;HOME:
            // Email3Address <-> EMAIL;INTERNET;WORK:
			if (vp->containsParameter("HOME")) {
				contactProperties[APPLE_CONTACT_EMAIL_2] = element;
			} else if (vp->containsParameter("WORK")) {
				contactProperties[APPLE_CONTACT_EMAIL_3] = element;
			} else {
				contactProperties[APPLE_CONTACT_EMAIL_1] = element;
            }
		} else if (strcmp(name, "FN") == 0) {
			contactProperties[APPLE_CONTACT_FILE_AS] = element;
        } else if (strcmp(name, "NICKNAME") == 0) {
			contactProperties[APPLE_CONTACT_NICKNAME] = element;
        } else if (strcmp(name, "ORG") == 0) {
            if (element = vp->getPropComponent(1))  contactProperties[APPLE_CONTACT_ORG] = element;
			if (element = vp->getPropComponent(2))  contactProperties[APPLE_CONTACT_ORG_UNIT] = element;
        } else if (strcmp(name, "TEL") == 0) {   // Telephone fields
		    if (vp->containsParameter("WORK")) {
                if (vp->containsParameter("FAX")) {
				    contactProperties[APPLE_CONTACT_PHONE_BUSINESS_FAX] = element;
                } else if (vp->containsParameter("PREF")) {
                    contactProperties[APPLE_CONTACT_PHONE_COMPANY] = element;
                } else if (vp->containsParameter("VOICE")) {
				    if (businessTel == 0) {
					    contactProperties[APPLE_CONTACT_PHONE_BUSINESS] = element;
					    businessTel++;
				    } else {
					    contactProperties[APPLE_CONTACT_PHONE_BUSINESS_2] = element;
				    }
                }
		    } else if (vp->containsParameter("CAR")) {
			    contactProperties[APPLE_CONTACT_PHONE_CAR] = element;
            } else if (vp->containsParameter("HOME")) {
                if (vp->containsParameter("FAX")) {
				    contactProperties[APPLE_CONTACT_PHONE_HOME_FAX] = element;
                } else if (vp->containsParameter("VOICE")) {
                    if (homeTel == 0) {
				        contactProperties[APPLE_CONTACT_PHONE_HOME] = element;
                        homeTel++;
                    } else {
				        contactProperties[APPLE_CONTACT_PHONE_HOME_2] = element;
                    }
                }
            } else if (vp->containsParameter("CELL")) {
			    contactProperties[APPLE_CONTACT_PHONE_MOBILE] = element;
            } else if (vp->containsParameter("PAGER")) {
			    contactProperties[APPLE_CONTACT_PHONE_PAGER] = element;
            } else if (vp->containsParameter("PREF")) {
                contactProperties[APPLE_CONTACT_PHONE_PRIMARY] = element;
            } else if (vp->containsParameter("VOICE")) {
			    contactProperties[APPLE_CONTACT_PHONE_OTHER] = element;
            }
        } else if(strcmp(name, "TITLE") == 0) {
			contactProperties[APPLE_CONTACT_TITLE] = element;
        } else if(strcmp(name, "PHOTO") == 0) {
            contactProperties[APPLE_CONTACT_PHOTO] = element; 
        } else if (strcmp(name, "URL") == 0) {
            if (vp->containsParameter("HOME")) {
			    contactProperties[APPLE_CONTACT_HOMEPAGE_URL] = element;
            } else {
                contactProperties[APPLE_CONTACT_URL] = element;
            }
        } else if (strcmp(name, "ADR") == 0) { // address fields
			if (vp->containsParameter("HOME")) {
				if (element = vp->getPropComponent(1))  contactProperties[APPLE_CONTACT_HOME_ADDRESS_PO] = element;
                if (element = vp->getPropComponent(2))  contactProperties[APPLE_CONTACT_HOME_ADDRESS_EXT] = element;
				if (element = vp->getPropComponent(3))  contactProperties[APPLE_CONTACT_HOME_ADDRESS_STREET] = element;
				if (element = vp->getPropComponent(4))  contactProperties[APPLE_CONTACT_HOME_ADDRESS_LOCALITY] = element;
				if (element = vp->getPropComponent(5))  contactProperties[APPLE_CONTACT_HOME_ADDRESS_REGION] = element;
				if (element = vp->getPropComponent(6))  contactProperties[APPLE_CONTACT_HOME_ADDRESS_CODE] = element;
				if (element = vp->getPropComponent(7))  contactProperties[APPLE_CONTACT_HOME_ADDRESS_COUNTRY] = element;
			} else if (vp->containsParameter("WORK")) {
				if (element = vp->getPropComponent(1))  contactProperties[APPLE_CONTACT_WORK_ADDRESS_PO] = element;
                if (element = vp->getPropComponent(2))  contactProperties[APPLE_CONTACT_WORK_ADDRESS_EXT] = element;
				if (element = vp->getPropComponent(3))  contactProperties[APPLE_CONTACT_WORK_ADDRESS_STREET] = element;
				if (element = vp->getPropComponent(4))  contactProperties[APPLE_CONTACT_WORK_ADDRESS_LOCALITY] = element;
				if (element = vp->getPropComponent(5))  contactProperties[APPLE_CONTACT_WORK_ADDRESS_REGION] = element;
				if (element = vp->getPropComponent(6))  contactProperties[APPLE_CONTACT_WORK_ADDRESS_CODE] = element;
				if (element = vp->getPropComponent(7))  contactProperties[APPLE_CONTACT_WORK_ADDRESS_COUNTRY] = element;
			} else {
                if (element = vp->getPropComponent(1))  contactProperties[APPLE_CONTACT_OTHER_ADDRESS_PO] = element;
                if (element = vp->getPropComponent(2))  contactProperties[APPLE_CONTACT_OTHER_ADDRESS_EXT] = element;
				if (element = vp->getPropComponent(3))  contactProperties[APPLE_CONTACT_OTHER_ADDRESS_STREET] = element;
				if (element = vp->getPropComponent(4))  contactProperties[APPLE_CONTACT_OTHER_ADDRESS_LOCALITY] = element;
				if (element = vp->getPropComponent(5))  contactProperties[APPLE_CONTACT_OTHER_ADDRESS_REGION] = element;
				if (element = vp->getPropComponent(6))  contactProperties[APPLE_CONTACT_OTHER_ADDRESS_CODE] = element;
				if (element = vp->getPropComponent(7))  contactProperties[APPLE_CONTACT_OTHER_ADDRESS_COUNTRY] = element;
			}
        } else if (strcmp(name, "X-ANNIVERSARY") == 0) { // extended properties
			contactProperties[APPLE_CONTACT_ANNIVERSARY] = element;
        } else if (strcmp(name, "X-MANAGER") == 0) {
            contactProperties[APPLE_CONTACT_MANAGER] = element;
        } else if (strcmp(name, "X-SPOUSE") == 0) {
            contactProperties[APPLE_CONTACT_SPOUSE] = element;
        }
    }
    
    delete vo;

    return true;
}

const char* AppleContactItem::formatIntoVCard(bool formatAllFields)
{
    VObject* vo = NULL; 
    VProperty* vp = NULL;
    string value;
    const char* vCard = NULL;

    vo = new VObject();

    vp = new VProperty(TEXT("BEGIN"), TEXT("VCARD"));
    vo->addProperty(vp);
    delete vp; vp = NULL;
        
    vp = new VProperty(TEXT("VERSION"), TEXT("2.1"));
    vo->addProperty(vp);
    delete vp; vp = NULL;

    // get all the available NAME fields
    vp = new VProperty(TEXT("N"));

    value = contactProperties[APPLE_CONTACT_FAMILY_NAME];
    if (value.empty() == false) {
        vp->addValue(value.c_str());
    } else {
        vp->addValue("");
    }

    value = contactProperties[APPLE_CONTACT_GIVEN_NAME];
    if (value.empty() == false) {
        vp->addValue(value.c_str());
    } else {
        vp->addValue("");
    }

    value = contactProperties[APPLE_CONTACT_ADDITIONAL_NAME];
    if (value.empty() == false) {
        vp->addValue(value.c_str());
    } else {
        vp->addValue("");
    }

    value = contactProperties[APPLE_CONTACT_NAME_PREFIXES];
    if (value.empty() == false) {
        vp->addValue(value.c_str());
    } else {
        vp->addValue("");
    }
 
    value = contactProperties[APPLE_CONTACT_NAME_SUFFIXES];
    if (value.empty() == false) {
        vp->addValue(value.c_str());
    } else {
        vp->addValue("");
    }

    vo->addProperty(vp);

    delete vp; vp = NULL;
 
    vp = new VProperty(TEXT("TITLE"));
    value = contactProperties[APPLE_CONTACT_TITLE];
    if (value.empty() == false) {  
        vp->addValue(value.c_str());
    } else {
        vp->addValue("");
    }
 
    vo->addProperty(vp);
    delete vp; vp = NULL;

    value = contactProperties[APPLE_CONTACT_BIRTH_DATE];
    if (value.empty() == false) {
        vp = new VProperty(TEXT("BDAY"), value.c_str());
        vo->addProperty(vp);

        delete vp; vp = NULL;
    } else {
        if (formatAllFields) {
            vp = new VProperty(TEXT("BDAY"), "");
            vo->addProperty(vp);

            delete vp; vp = NULL;
        }
    }

    value = contactProperties[APPLE_CONTACT_ANNIVERSARY];
    if (value.empty() == false) {
        vp = new VProperty(TEXT("X-ANNIVERSARY"), value.c_str());
        vo->addProperty(vp);

        delete vp; vp = NULL;
    } else {
        if (formatAllFields) {
            vp = new VProperty(TEXT("X-ANNIVERSARY"), "");
            vo->addProperty(vp);

            delete vp; vp = NULL;
        }
    }

    value = contactProperties[APPLE_CONTACT_NOTE];    
    if (value.empty() == false) {
        vp = new VProperty(TEXT("NOTE"), value.c_str());
        vo->addProperty(vp);
        delete vp; vp = NULL;
    } else {
        if (formatAllFields) {
            vp = new VProperty(TEXT("NOTE"), "");
            vo->addProperty(vp);

            delete vp; vp = NULL;
        }
    }


    // Telephone numbers

    // first work phone
    value = contactProperties[APPLE_CONTACT_PHONE_BUSINESS];
    if (value.empty() == false) {
        vp = new VProperty(TEXT("TEL"), value.c_str());
        vp->addParameter(TEXT("VOICE"), NULL);
        vp->addParameter(TEXT("WORK"),  NULL);
        vo->addProperty(vp);
        delete vp; vp = NULL;
    } else {
        if (formatAllFields) {
            vp = new VProperty(TEXT("TEL"), "");
            vp->addParameter(TEXT("VOICE"), NULL);
            vp->addParameter(TEXT("WORK"),  NULL);
            vo->addProperty(vp);
            delete vp; vp = NULL;
        }
    }

    value = contactProperties[APPLE_CONTACT_PHONE_BUSINESS_2];
    if (value.empty() == false) {
        vp = new VProperty(TEXT("TEL"), value.c_str());
        vp->addParameter(TEXT("VOICE"), NULL);
        vp->addParameter(TEXT("WORK"),  NULL);
        vo->addProperty(vp);
        delete vp; vp = NULL;
    } else {
        if (formatAllFields) {
            vp = new VProperty(TEXT("TEL"), "");
            vp->addParameter(TEXT("VOICE"), NULL);
            vp->addParameter(TEXT("WORK"),  NULL);
            vo->addProperty(vp);
            delete vp; vp = NULL;
        }
    }
    
    value = contactProperties[APPLE_CONTACT_PHONE_BUSINESS_FAX];
    if (value.empty() == false) {
        vp = new VProperty(TEXT("TEL"), value.c_str());
        
        vp->addParameter(TEXT("WORK"), NULL);
        vp->addParameter(TEXT("FAX"), NULL);
        
        vo->addProperty(vp);
        
        delete vp; vp = NULL;
    } else {
        if (formatAllFields) {
            vp = new VProperty(TEXT("TEL"), "");
        
            vp->addParameter(TEXT("WORK"), NULL);
            vp->addParameter(TEXT("FAX"), NULL);
        
            vo->addProperty(vp);
        
            delete vp; vp = NULL;
        }
    }

    // company tel
    value = contactProperties[APPLE_CONTACT_PHONE_COMPANY];
    if (value.empty() == false) {
        vp = new VProperty(TEXT("TEL"), value.c_str());
        
        vp->addParameter(TEXT("WORK"), NULL);
        vp->addParameter(TEXT("PREF"), NULL);
        
        vo->addProperty(vp);
        
        delete vp; vp = NULL;
    } else {
        if (formatAllFields) {
            vp = new VProperty(TEXT("TEL"), "");
        
            vp->addParameter(TEXT("WORK"), NULL);
            vp->addParameter(TEXT("PREF"), NULL);
        
            vo->addProperty(vp);
        
            delete vp; vp = NULL;
        }
    }
    
    // home tel
    value = contactProperties[APPLE_CONTACT_PHONE_HOME];
    if (value.empty() == false) {
        vp = new VProperty(TEXT("TEL"), value.c_str());
       
        vp->addParameter(TEXT("VOICE"), NULL);
        vp->addParameter(TEXT("HOME"),  NULL); 
        
        vo->addProperty(vp);
        
        delete vp; vp = NULL;
    } else {
        if (formatAllFields) {
            vp = new VProperty(TEXT("TEL"), "");
       
            vp->addParameter(TEXT("VOICE"), NULL);
            vp->addParameter(TEXT("HOME"),  NULL); 
        
            vo->addProperty(vp);
        
            delete vp; vp = NULL;
        }
    } 

    // home tel 2
    value = contactProperties[APPLE_CONTACT_PHONE_HOME_2];
    if (value.empty() == false) {
        vp = new VProperty(TEXT("TEL"), value.c_str());
       
        vp->addParameter(TEXT("VOICE"), NULL);
        vp->addParameter(TEXT("HOME"),  NULL); 
        
        vo->addProperty(vp);
        
        delete vp; vp = NULL;
    } else {
        if (formatAllFields) {
            vp = new VProperty(TEXT("TEL"), "");
       
            vp->addParameter(TEXT("VOICE"), NULL);
            vp->addParameter(TEXT("HOME"),  NULL); 
        
            vo->addProperty(vp);
        
            delete vp; vp = NULL;
        }
    }

    value = contactProperties[APPLE_CONTACT_PHONE_HOME_FAX];
    if (value.empty() == false) {
        vp = new VProperty(TEXT("TEL"), value.c_str());
      
        vp->addParameter(TEXT("HOME"), NULL);
        vp->addParameter(TEXT("FAX"), NULL);
 
        vo->addProperty(vp);
        
        delete vp; vp = NULL;
    } else {
        if (formatAllFields) {
            vp = new VProperty(TEXT("TEL"), "");
      
            vp->addParameter(TEXT("HOME"), NULL);
            vp->addParameter(TEXT("FAX"), NULL);
 
            vo->addProperty(vp);
        
            delete vp; vp = NULL;
        }
    }

    value = contactProperties[APPLE_CONTACT_PHONE_CAR];
    if (value.empty() == false) {
        vp = new VProperty(TEXT("TEL"), value.c_str());
      
        vp->addParameter(TEXT("CAR"), NULL); 
        vo->addProperty(vp);
        
        delete vp; vp = NULL;
    } else {
        if (formatAllFields) {
            vp = new VProperty(TEXT("TEL"), "");
      
            vp->addParameter(TEXT("CAR"), NULL); 
            vo->addProperty(vp);
        
            delete vp; vp = NULL;
        }
    }

    value = contactProperties[APPLE_CONTACT_PHONE_MOBILE];
    if (value.empty() == false) {
        vp = new VProperty(TEXT("TEL"), value.c_str());
      
        vp->addParameter(TEXT("CELL"), NULL); 
        vo->addProperty(vp);
        
        delete vp; vp = NULL;
    } else {
        if (formatAllFields) {
            vp = new VProperty(TEXT("TEL"), "");
            vp->addParameter(TEXT("CELL"), NULL); 
            vo->addProperty(vp);
        
            delete vp; vp = NULL;
        }
    }
   
    value = contactProperties[APPLE_CONTACT_PHONE_OTHER];
    if (value.empty() == false) {
        vp = new VProperty(TEXT("TEL"), value.c_str());
      
        vp->addParameter(TEXT("VOICE"), NULL); 
        vo->addProperty(vp);
        
        delete vp; vp = NULL;
    } else {
        if (formatAllFields) {
            vp = new VProperty(TEXT("TEL"), "");
      
            vp->addParameter(TEXT("VOICE"), NULL); 
            vo->addProperty(vp);
        
            delete vp; vp = NULL;
        }
    }
    
    value = contactProperties[APPLE_CONTACT_PHONE_PAGER];
    if (value.empty() == false) {
        vp = new VProperty(TEXT("TEL"), value.c_str());
      
        vp->addParameter(TEXT("PAGER"), NULL); 
        vo->addProperty(vp);
        
        delete vp; vp = NULL;
    }  else {
        if (formatAllFields) {
            vp = new VProperty(TEXT("TEL"), "");
      
            vp->addParameter(TEXT("PAGER"), NULL); 
            vo->addProperty(vp);
        
            delete vp; vp = NULL;
        }
    }

    value = contactProperties[APPLE_CONTACT_PHONE_PRIMARY];
    if (value.empty() == false) {
        vp = new VProperty(TEXT("TEL"), value.c_str());
     
        vp->addParameter(TEXT("PREF"),  NULL);
        vp->addParameter(TEXT("VOICE"), NULL);
 
        vo->addProperty(vp);
        
        delete vp; vp = NULL;
    } else {
        if (formatAllFields) {
            vp = new VProperty(TEXT("TEL"), "");
     
            vp->addParameter(TEXT("PREF"),  NULL);
            vp->addParameter(TEXT("VOICE"), NULL);
 
            vo->addProperty(vp);
        
            delete vp; vp = NULL;
        }
    }
   
    value = contactProperties[APPLE_CONTACT_HOMEPAGE_URL];
    if (value.empty() == false) {
        vp = new VProperty(TEXT("URL"), value.c_str());
     
        vp->addParameter(TEXT("HOME"),  NULL);
 
        vo->addProperty(vp);
        
        delete vp; vp = NULL;
    } else {
        if (formatAllFields) {
            vp = new VProperty(TEXT("URL"), "");
     
            vp->addParameter(TEXT("HOME"),  NULL);
 
            vo->addProperty(vp);
        
            delete vp; vp = NULL;
        }
    }

    value = contactProperties[APPLE_CONTACT_URL];
    if (value.empty() == false) {
        vp = new VProperty(TEXT("URL"), value.c_str());
        vo->addProperty(vp);
        
        delete vp; vp = NULL;
    } else {
        if (formatAllFields) {
            vp = new VProperty(TEXT("URL"), "");
            vo->addProperty(vp);
        
            delete vp; vp = NULL;
        }
    }
    
    value = contactProperties[APPLE_CONTACT_CATEGORIES];
    if (value.empty() == false) {
        vp = new VProperty(TEXT("CATEGORIES"), value.c_str());
     
        vo->addProperty(vp);
        
        delete vp; vp = NULL;
    } else {
        if (formatAllFields) {
            vp = new VProperty(TEXT("CATEGORIES"), "");
     
            vo->addProperty(vp);
        
            delete vp; vp = NULL;
        }
    }

    value = contactProperties[APPLE_CONTACT_FILE_AS];
    if (value.empty() == false) {
        vp = new VProperty(TEXT("FN"), value.c_str());
     
        vo->addProperty(vp);
        
        delete vp; vp = NULL;
    } else {
        if (formatAllFields) {
            vp = new VProperty(TEXT("FN"), "");
     
            vo->addProperty(vp);
        
            delete vp; vp = NULL;
        }
    }

    value = contactProperties[APPLE_CONTACT_EMAIL_1];
    if (value.empty() == false) {
        vp = new VProperty(TEXT("EMAIL"), value.c_str()); 

        vp->addParameter(TEXT("INTERNET"), NULL);
        vo->addProperty(vp);
        
        delete vp; vp = NULL;
    } else {
        if (formatAllFields) {
            vp = new VProperty(TEXT("EMAIL"), ""); 

            vp->addParameter(TEXT("INTERNET"), NULL);
            vo->addProperty(vp);
        
            delete vp; vp = NULL;
        }
    }

    value = contactProperties[APPLE_CONTACT_EMAIL_2];
    if (value.empty() == false) {
        vp = new VProperty(TEXT("EMAIL"), value.c_str());
        
        vp->addParameter(TEXT("INTERNET"), NULL);
        vp->addParameter(TEXT("HOME"),     NULL);
        vo->addProperty(vp);
        
        delete vp; vp = NULL;
    } else {
        if (formatAllFields) {
            vp = new VProperty(TEXT("EMAIL"), value.c_str());
        
            vp->addParameter(TEXT("INTERNET"), NULL);
            vp->addParameter(TEXT("HOME"),     NULL);
            vo->addProperty(vp);
        
            delete vp; vp = NULL;
        }
    }

    value = contactProperties[APPLE_CONTACT_EMAIL_3];
    if (value.empty() == false) {
        vp = new VProperty(TEXT("EMAIL"), value.c_str());

        vp->addParameter(TEXT("INTERNET"), NULL);
        vp->addParameter(TEXT("WORK"),     NULL);
        vo->addProperty(vp);

        delete vp; vp = NULL;
    } else {
        if (formatAllFields) {
            vp = new VProperty(TEXT("EMAIL"), "");

            vp->addParameter(TEXT("INTERNET"), NULL);
            vp->addParameter(TEXT("WORK"),     NULL);
            vo->addProperty(vp);

            delete vp; vp = NULL;
        }
    }


    value = contactProperties[APPLE_CONTACT_MANAGER];
    if (value.empty() == false) {
        vp = new VProperty(TEXT("X-MANAGER"), value.c_str());
        vo->addProperty(vp);
        delete vp; vp = NULL;
    } else {
        if (formatAllFields) {
            vp = new VProperty(TEXT("X-MANAGER"), "");
            vo->addProperty(vp);
            delete vp; vp = NULL;
        }
    }

    value = contactProperties[APPLE_CONTACT_ASSISTANT];
    if (value.empty() == false) {
        vp = new VProperty(TEXT("X-ASSISTANT"), value.c_str());
        vo->addProperty(vp);
        delete vp; vp = NULL;
    } else {
        if (formatAllFields) {
            vp = new VProperty(TEXT("X-ASSISTANT"), "");
            vo->addProperty(vp);
            delete vp; vp = NULL;
        }
    }

    value = contactProperties[APPLE_CONTACT_SPOUSE];
    if (value.empty() == false) {
        vp = new VProperty(TEXT("X-SPOUSE"), value.c_str());
        vo->addProperty(vp);
        delete vp; vp = NULL;
    } else {
        if (formatAllFields) {
            vp = new VProperty(TEXT("X-SPOUSE"), "");
            vo->addProperty(vp);
            delete vp; vp = NULL;
        }
    }

    value = contactProperties[APPLE_CONTACT_NICKNAME];
    if (value.empty() == false) {
        vp = new VProperty(TEXT("NICKNAME"), value.c_str());
        vo->addProperty(vp);
        delete vp; vp = NULL;
    } else {
        if (formatAllFields) {
            vp = new VProperty(TEXT("NICKNAME"), "");
            vo->addProperty(vp);
            delete vp; vp = NULL;
        }
    }


    vp = new VProperty(TEXT("ORG"));
    value = contactProperties[APPLE_CONTACT_ORG];
    if (value.empty() == false) {
        vp->addValue(value.c_str());
    } else {
        vp->addValue("");
    }

    value = contactProperties[APPLE_CONTACT_ORG_UNIT];
    if (value.empty() == false) {
        vp->addValue(value.c_str());
    } else {
        vp->addValue("");
    }

    vo->addProperty(vp);
    delete vp; vp = NULL;

    // home address
    vp = new VProperty(TEXT("ADR"));
    vp->addParameter(TEXT("HOME"), NULL);
    
    value = contactProperties[APPLE_CONTACT_HOME_ADDRESS_PO];
    if (value.empty() == false) {
        vp->addValue(value.c_str());
    } else {
        vp->addValue("");
    }

    value = contactProperties[APPLE_CONTACT_HOME_ADDRESS_EXT];
    if (value.empty() == false) {
        vp->addValue(value.c_str());
    } else {
        vp->addValue("");
    }

    value = contactProperties[APPLE_CONTACT_HOME_ADDRESS_STREET];
    if (value.empty() == false) {
        vp->addValue(value.c_str());
    } else {
        vp->addValue("");
    }

    value = contactProperties[APPLE_CONTACT_HOME_ADDRESS_LOCALITY];
    if (value.empty() == false) {
        vp->addValue(value.c_str());
    } else {
        vp->addValue("");
    }

    value = contactProperties[APPLE_CONTACT_HOME_ADDRESS_REGION];
    if (value.empty() == false) {
        vp->addValue(value.c_str());
    } else {
        vp->addValue(""); 
    }

    value = contactProperties[APPLE_CONTACT_HOME_ADDRESS_CODE];
    if (value.empty() == false) {
        vp->addValue(value.c_str());
    } else {
        vp->addValue("");
    }

    value = contactProperties[APPLE_CONTACT_HOME_ADDRESS_COUNTRY];
    if (value.empty() == false) {
        vp->addValue(value.c_str());
    } else {
        vp->addValue("");
    }
    
    vo->addProperty(vp);
    delete vp; vp = NULL;

    // other address
    vp = new VProperty(TEXT("ADR"));
    value = contactProperties[APPLE_CONTACT_OTHER_ADDRESS_PO];
    if (value.empty() == false) {
        vp->addValue(value.c_str());
    } else {
        vp->addValue("");
    }

    value = contactProperties[APPLE_CONTACT_OTHER_ADDRESS_EXT];
    if (value.empty() == false) {
        vp->addValue(value.c_str());
    } else {
        vp->addValue("");
    }

    value = contactProperties[APPLE_CONTACT_OTHER_ADDRESS_STREET];
    if (value.empty() == false) {
        vp->addValue(value.c_str());
    } else {
        vp->addValue("");
    }

    value = contactProperties[APPLE_CONTACT_OTHER_ADDRESS_LOCALITY];
    if (value.empty() == false) {
        vp->addValue(value.c_str());
    } else {
        vp->addValue("");
    }

    value = contactProperties[APPLE_CONTACT_OTHER_ADDRESS_REGION];
    if (value.empty() == false) {
        vp->addValue(value.c_str());
    } else {
        vp->addValue("");
    }

    value = contactProperties[APPLE_CONTACT_OTHER_ADDRESS_CODE];
    if (value.empty() == false) {
        vp->addValue(value.c_str());
    } else {
        vp->addValue("");
    }

    value = contactProperties[APPLE_CONTACT_OTHER_ADDRESS_COUNTRY];
    if (value.empty() == false) {
        vp->addValue(value.c_str());
    } else {
        vp->addValue("");
    }
    vo->addProperty(vp);
    delete vp; vp = NULL;

    // work address
    vp = new VProperty(TEXT("ADR"));
    vp->addParameter(TEXT("WORK"), NULL);
    
    value = contactProperties[APPLE_CONTACT_WORK_ADDRESS_PO];
    if (value.empty() == false) {
        vp->addValue(value.c_str());
    } else {
        vp->addValue("");
    }

    value = contactProperties[APPLE_CONTACT_WORK_ADDRESS_EXT];
    if (value.empty() == false) {
        vp->addValue(value.c_str());
    } else {
        vp->addValue("");
    }

    value = contactProperties[APPLE_CONTACT_WORK_ADDRESS_STREET];
    if (value.empty() == false) {
        vp->addValue(value.c_str());
    } else {
        vp->addValue("");
    }

    value = contactProperties[APPLE_CONTACT_WORK_ADDRESS_LOCALITY];
    if (value.empty() == false) {
        vp->addValue(value.c_str());
    } else {
        vp->addValue("");
    }

    value = contactProperties[APPLE_CONTACT_WORK_ADDRESS_REGION];
    if (value.empty() == false) {
        vp->addValue(value.c_str());
    } else {
        vp->addValue("");
    }

    value = contactProperties[APPLE_CONTACT_WORK_ADDRESS_CODE];
    if (value.empty() == false) {
        vp->addValue(value.c_str());
    } else {
        vp->addValue("");
    }

    value = contactProperties[APPLE_CONTACT_WORK_ADDRESS_COUNTRY];
    if (value.empty() == false) {
        vp->addValue(value.c_str());
    } else {
        vp->addValue("");   
    }

    vo->addProperty(vp);
    delete vp; vp = NULL;

    value = contactProperties[APPLE_CONTACT_PHOTO];
    
    if (value.empty() == false) {
        int valueLen = value.size();
        char valueDecoded[valueLen];
        memset(valueDecoded, 0, valueLen);
     
        int decodedLen = b64_decode(valueDecoded, value.c_str());
        char *contentEncoded = b64EncodeWithSpaces(valueDecoded, decodedLen);
        vp = new VProperty("PHOTO", contentEncoded);
    
        vp->addParameter("ENCODING", "b");
        vp->addParameter("TYPE", "JPEG");
        vp->addParameter("CONTENT-VALUE", "UNCHANGED");
    } else { 
        vp = new VProperty("PHOTO", "");
    }

    vo->addProperty(vp);
           
    vp = new VProperty(TEXT("END"), TEXT("VCARD"));
    vo->addProperty(vp);
    delete vp; vp = NULL;

    // format vCard.
    vCard = vo->toString();
    
    delete vo;

    return vCard;
}

const char* AppleContactItem::getPropertyValue(AppleContactItemPropertyId propId) const
{
    const char *propVal = NULL;

    if (contactProperties.size()) {
        map<AppleContactItemPropertyId, string>::const_iterator it = contactProperties.find(propId);

        if (it != contactProperties.end()) {
            propVal = static_cast<string>(it->second).c_str();
        }
    }

    return propVal;
}
