/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2010 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */


#ifndef __APPLE_CONTACT_ITEM_H__
#define __APPLE_CONTACT_ITEM_H__

#include <map>
#include <string>
#include "vocl/VConverter.h"
#include "spds/SyncItem.h"
#include "base/util/StringBuffer.h"
#include "base/Log.h"

// supported contact fields
typedef enum {
    APPLE_CONTACT_FAMILY_NAME = 0,
    APPLE_CONTACT_GIVEN_NAME,
    APPLE_CONTACT_ADDITIONAL_NAME,
    APPLE_CONTACT_NAME_PREFIXES,
    APPLE_CONTACT_NAME_SUFFIXES,

    APPLE_CONTACT_TITLE,
    APPLE_CONTACT_BIRTH_DATE,
    APPLE_CONTACT_ANNIVERSARY,
    APPLE_CONTACT_NOTE,

    APPLE_CONTACT_PHONE_BUSINESS,
    APPLE_CONTACT_PHONE_BUSINESS_2,
    APPLE_CONTACT_PHONE_BUSINESS_FAX,
    APPLE_CONTACT_PHONE_COMPANY,
    APPLE_CONTACT_PHONE_HOME,
    APPLE_CONTACT_PHONE_HOME_2,
    APPLE_CONTACT_PHONE_HOME_FAX,
    APPLE_CONTACT_PHONE_CAR,
    APPLE_CONTACT_PHONE_MOBILE,
    APPLE_CONTACT_PHONE_OTHER,
    APPLE_CONTACT_PHONE_PAGER,
    APPLE_CONTACT_PHONE_PRIMARY,

    APPLE_CONTACT_HOMEPAGE_URL,
    APPLE_CONTACT_URL,
    APPLE_CONTACT_CATEGORIES,
    APPLE_CONTACT_FILE_AS,

    APPLE_CONTACT_EMAIL_1,
    APPLE_CONTACT_EMAIL_2,
    APPLE_CONTACT_EMAIL_3,

    APPLE_CONTACT_MANAGER,
    APPLE_CONTACT_ASSISTANT,
    APPLE_CONTACT_SPOUSE,
    APPLE_CONTACT_NICKNAME,
    APPLE_CONTACT_ORG,
    APPLE_CONTACT_ORG_UNIT,

    // home address fields
    APPLE_CONTACT_HOME_ADDRESS_PO,
    APPLE_CONTACT_HOME_ADDRESS_EXT,
    APPLE_CONTACT_HOME_ADDRESS_STREET,
    APPLE_CONTACT_HOME_ADDRESS_LOCALITY,
    APPLE_CONTACT_HOME_ADDRESS_REGION,
    APPLE_CONTACT_HOME_ADDRESS_CODE,
    APPLE_CONTACT_HOME_ADDRESS_COUNTRY,
    // work address fields
    APPLE_CONTACT_WORK_ADDRESS_PO,
    APPLE_CONTACT_WORK_ADDRESS_EXT,
    APPLE_CONTACT_WORK_ADDRESS_STREET,
    APPLE_CONTACT_WORK_ADDRESS_LOCALITY,
    APPLE_CONTACT_WORK_ADDRESS_REGION,
    APPLE_CONTACT_WORK_ADDRESS_CODE,
    APPLE_CONTACT_WORK_ADDRESS_COUNTRY,
    // other address
    APPLE_CONTACT_OTHER_ADDRESS_PO,
    APPLE_CONTACT_OTHER_ADDRESS_EXT,
    APPLE_CONTACT_OTHER_ADDRESS_STREET,
    APPLE_CONTACT_OTHER_ADDRESS_LOCALITY,
    APPLE_CONTACT_OTHER_ADDRESS_REGION,
    APPLE_CONTACT_OTHER_ADDRESS_CODE,
    APPLE_CONTACT_OTHER_ADDRESS_COUNTRY,

    APPLE_CONTACT_PHOTO
} AppleContactItemPropertyId;

class AppleContactItem
{
    private:
        std::map<AppleContactItemPropertyId, std::string> contactProperties;

    public:
        AppleContactItem();
        AppleContactItem(const char* vCard);
        
        bool initFromVCard(const char* vCard);
        const char* formatIntoVCard(bool formatAllFields = false);
        
        void setPropertyValue(const char* propertyName, const char* propertyValue);
        const char* getPropertyValue(AppleContactItemPropertyId propId) const;
};

#endif
