/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2010 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include "AddressBookSyncSourceTest.h"

USE_FUNAMBOL_NAMESPACE

const char* AddressBookSyncSourceTest::testName = "AddressBookSyncSourceTest";
const char* AddressBookSyncSourceTest::testVCard  = 
"BEGIN:VCARD\r\n"
"VERSION:2.1\r\n"
"N:John;Doe;Johnny;;\r\n"
"TITLE: C++ developer\r\n"
"BDAY:\r\n"
"X-ANNIVERSARY:\r\n"
"NOTE:test note\r\n"
"TEL;VOICE;WORK:328489288\r\n"
"TEL;VOICE;WORK:328489288\r\n"
"TEL;WORK;FAX:328489288\r\n"
"TEL;WORK;PREF:0849389488\r\n"
"TEL;VOICE;HOME:123134\r\n"
"TEL;VOICE;HOME:\r\n"
"TEL;HOME;FAX:12390909\r\n"
"TEL;CAR:\r\n"
"TEL;CELL:234-1209\r\n"
"TEL;VOICE:12313\r\n"
"TEL;PAGER:3894989898\r\n"
"TEL;PREF;VOICE:23423\r\n"
"URL;HOME:\r\n"
"CATEGORIES:\r\n"
"FN:\r\n"
"EMAIL;INTERNET:john@doe.org\r\n"
"EMAIL;INTERNET;HOME:johndoe@home.it\r\n"
"EMAIL;INTERNET;WORK:johndoe@work.com\r\n"
"X-MANAGER:\r\n"
"X-ASSISTANT:\r\n"
"X-SPOUSE:\r\n"
"NICKNAME:\r\n"
"ORG:Funambol;engineering\r\n"
"ADR;HOME:;;via cavour;Pavia;PV;27043;IT\r\n"
"ADR:;;via caracciolo;milan;MI;9123;IT\r\n"
"ADR;WORK:;;via riviera 55;Pavia;PV;27100;IT\r\n"
"PHOTO:\r\n"
"END:VCARD\r\n";

const char* AddressBookSyncSourceTest::testBDAYVCard  = 
"BEGIN:VCARD\r\n"
"VERSION:2.1\r\n"
"N:Stanely;Stan;Kubrick;;\r\n"
"TITLE:film director\r\n"
"BDAY:1928-07-26\r\n"
"END:VCARD\r\n";

void AddressBookSyncSourceTest::setUp() 
{
    const char* source_name = "addressbook";

    PlatformAdapter::init(testName);
    config = UtClientConfig::getInstance(); 
    sc = config->getSyncSourceConfig(source_name, true);
    abss = new AddressBookSyncSource(source_name, sc);
}

void AddressBookSyncSourceTest::testFieldLevelReplace()
{
    const char* itemId = "1234567890";
    StringBuffer vCardFromAB = NULL;
    const char* vCardField  = NULL;
    SyncItem* syncItem = new SyncItem(itemId);
    int res = STC_ITEM_ADDED;
    size_t len = 0;
    StringBuffer vCardString(testVCard);
    StringBuffer itemKey;
    AppleContactItem *contactItem = NULL;
    
    syncItem->setData(vCardString.c_str(), (vCardString.length() * sizeof(char)));
    LOG.info("vcard data set into sync item: %s", vCardString.c_str());
    res = abss->insertItem(*syncItem);
    CPPUNIT_ASSERT(res != STC_COMMAND_FAILED);

    itemKey = syncItem->getKey();
    vCardFromAB = (const char *)abss->getItemContent(itemKey, &len);
    CPPUNIT_ASSERT(vCardFromAB != NULL);
    CPPUNIT_ASSERT(len != 0);

    LOG.info("removing 'TITLE' and 'EMAIL;INTERNET;HOME:' fields from vcard");
    // remove some field from vCard
    vCardString = removeFieldFromVCard(vCardString, "TITLE:");
    vCardString = removeFieldFromVCard(vCardString, "EMAIL;INTERNET;HOME:");
    
    contactItem = new AppleContactItem(vCardString.c_str());
    // format vcard with all fields
    LOG.info("formatting new vcard with field level replace enabled");
    vCardString = contactItem->formatIntoVCard(true);
    delete contactItem;

    syncItem->setData(vCardString.c_str(), (vCardString.length() * sizeof(char)));

    res = abss->modifyItem(*syncItem);
    CPPUNIT_ASSERT(res != STC_COMMAND_FAILED);

    vCardFromAB = (const char *)abss->getItemContent(itemKey, &len);
    LOG.info("vcard from sync item: %s", vCardFromAB.c_str());
    CPPUNIT_ASSERT(vCardFromAB != NULL);
    CPPUNIT_ASSERT(len != 0);

    vCardField = getFieldFromVCard(vCardFromAB, "TITLE:");
    CPPUNIT_ASSERT(vCardField == NULL);
    
    vCardField = getFieldFromVCard(vCardFromAB, "EMAIL;INTERNET;HOME:");
    CPPUNIT_ASSERT(vCardField == NULL);
    
    res = abss->removeItem(*syncItem);
    CPPUNIT_ASSERT(res != STC_COMMAND_FAILED);
} 

void AddressBookSyncSourceTest::testContactBdayFieldLevelReplace()
{
    const char* itemId = "abcdefgh";
    StringBuffer vCardFromAB = NULL;
    const char* vCardField  = NULL;
    SyncItem* syncItem = new SyncItem(itemId);
    int res = STC_ITEM_ADDED;
    size_t len = 0;
    StringBuffer vCardString(testBDAYVCard);
    StringBuffer itemKey;
    AppleContactItem *contactItem = NULL;
    
    syncItem->setData(vCardString.c_str(), (vCardString.length() * sizeof(char)));
    LOG.info("vcard data set into sync item: %s", vCardString.c_str());
    res = abss->insertItem(*syncItem);
    CPPUNIT_ASSERT(res != STC_COMMAND_FAILED);

    itemKey = syncItem->getKey();
    vCardFromAB = (const char *)abss->getItemContent(itemKey, &len);
    CPPUNIT_ASSERT(vCardFromAB != NULL);
    CPPUNIT_ASSERT(len != 0);

    LOG.info("removing 'BDAY' field from vcard");
    // remove some field from vCard
    vCardString = removeFieldFromVCard(vCardString, "BDAY:");
        
    contactItem = new AppleContactItem(vCardString.c_str());
    // format vcard with all fields
    LOG.info("formatting new vcard with field level replace enabled");
    vCardString = contactItem->formatIntoVCard(true);
    delete contactItem;
    
    syncItem->setData(vCardString.c_str(), (vCardString.length() * sizeof(char)));

    res = abss->modifyItem(*syncItem);
    CPPUNIT_ASSERT(res != STC_COMMAND_FAILED);

    vCardFromAB = (const char *)abss->getItemContent(itemKey, &len);
    LOG.info("vcard from sync item: %s", vCardFromAB.c_str());
    CPPUNIT_ASSERT(vCardFromAB != NULL);
    CPPUNIT_ASSERT(len != 0);

    vCardField = getFieldFromVCard(vCardFromAB, "BDAY:");
    CPPUNIT_ASSERT(vCardField == NULL);
    
    res = abss->removeItem(*syncItem);
    CPPUNIT_ASSERT(res != STC_COMMAND_FAILED);
} 


const char* AddressBookSyncSourceTest::getFieldFromVCard(StringBuffer& vCard, const char* fieldName)
{
    size_t fieldStart = StringBuffer::npos;
    const char* fieldValue = NULL;

    if ((fieldName == NULL) || (vCard.empty())) {
        return vCard;
    }

    if ((fieldStart = vCard.ifind(fieldName)) != StringBuffer::npos) {
        size_t endOfFieldLabel = vCard.find(":", fieldStart);
        size_t endOfField = vCard.find("\n", fieldStart);

        if (endOfField != StringBuffer::npos) {
            fieldValue = vCard.substr(endOfFieldLabel + 1, (endOfField - endOfFieldLabel));
            if (strcmp(fieldValue, "\r\n") == 0) { // reset value if field has no entry
                fieldValue = NULL;
            }
        }
    }

    return fieldValue;
}


const char* AddressBookSyncSourceTest::removeFieldFromVCard(StringBuffer& vCard, const char* fieldName)
{
    size_t fieldStart = StringBuffer::npos;

    if ((fieldName == NULL) || (vCard.empty())) {
        return vCard;
    }

    if ((fieldStart = vCard.ifind(fieldName)) != StringBuffer::npos) {
        size_t endOfField = vCard.find("\n", fieldStart);

        if (endOfField != StringBuffer::npos) {
            StringBuffer titleField = vCard.substr(fieldStart, (endOfField - fieldStart) + 1);

            vCard.replace(titleField, "", fieldStart);
        }

        LOG.debug("vcard after replacement: %s", vCard.c_str());
    }

    return vCard.c_str();
}

CPPUNIT_TEST_SUITE_REGISTRATION(AddressBookSyncSourceTest);
