/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2010 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include "UtClientConfig.h"
#include "MacDefaultConfigFactory.h"

USE_FUNAMBOL_NAMESPACE

UtClientConfig* UtClientConfig::instance = NULL;

UtClientConfig* UtClientConfig::getInstance()
{
    if (instance == NULL) {
        instance = new UtClientConfig();

        if (instance->init() == false) {
            delete instance;
            instance = NULL;
        }
    }

    return instance;
}

UtClientConfig::UtClientConfig() : DMTClientConfig() {}

bool UtClientConfig::init() 
{
    bool ret = true;
    
    if ((ret = read()) == false) {
        // config not found, generate a default one
        createConfig();
        if (AddressBookSyncSource::initializeDatabase()){
            LOG.error("AddressBookSyncSource::initializeDatabase failed - %s", getLastErrorMsg());
            ret = false;
        } else {
            ret = read();
        }
    } else {
        if (AddressBookSyncSource::verifyDatabase()){
            LOG.error("AddressBookSyncSource::verifyDatabase failed - %s", getLastErrorMsg());
            ret = false;
        }
    }
    
    return ret;
}

void UtClientConfig::createConfig() 
{
    setAccessConfig(* MacDefaultConfigFactory::getAccessConfig());
    setDeviceConfig(* MacDefaultConfigFactory::getDeviceConfig());    
    setSyncSourceConfig(* MacDefaultConfigFactory::getSyncSourceConfig("addressbook"));

    save();
}
